##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Adapters for the `zc.displayname` package.

$Id: adapters.py 4811 2006-01-17 20:45:52Z fred $
"""

from zope import interface, component, i18n, proxy
from zope.i18nmessageid import Message
from zope.security.interfaces import Unauthorized
import zope.dublincore.interfaces
import zope.location.interfaces
from zope.publisher.interfaces import IRequest
from zope.publisher.interfaces.http import IHTTPRequest
from zope.publisher.browser import BrowserView
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.traversing.interfaces import IContainmentRoot

from zc.displayname import interfaces
from zc.displayname.i18n import _

INSUFFICIENT_CONTEXT = _("There isn't enough context to get URL information. "
                       "This is probably due to a bug in setting up location "
                       "information.")

class DefaultDisplayNameGenerator(BrowserView):
    component.adapts(zope.location.interfaces.ILocation, IRequest)
    interface.implementsOnly(interfaces.IDisplayNameGenerator)

    def __call__(self, maxlength=None):
        ob = self.context
        try:
            try:
                dc = zope.dublincore.interfaces.IDCDescriptiveProperties(ob)
            except TypeError:
                name = ob.__name__
            else:
                name = dc.title or ob.__name__
        except Unauthorized:
            name = ob.__name__ # __name__ should always be available; if it is
            # not, we consider that a configuration error.
        return convertName(name, self.request, maxlength)

class SiteDisplayNameGenerator(BrowserView):
    component.adapts(IContainmentRoot, IRequest)
    interface.implementsOnly(interfaces.IDisplayNameGenerator)

    def __call__(self, maxlength=None):
        return convertName(_('[root]'), self.request, maxlength)

def convertName(name, request, maxlength):
    """trim name to maxlength, if given.  Translate, if appropriate.

    Not appropriate for names with HTML.
    """
    if isinstance(name, Message):
        name = i18n.translate(name, context=request, default=name)
    if maxlength is not None:
        if not isinstance(maxlength, (int, long)):
            raise TypeError('maxlength must be int', maxlength)
        if maxlength < 0:
            raise ValueError('maxlength must be 0 or greater', maxlength)
        name_len = len(name)
        if name_len > maxlength:
            if maxlength < 4:
                name = '?' * maxlength
            else:
                name = name[:maxlength-3] + "..."
    return name

class Breadcrumbs(BrowserView):
    interface.implementsOnly(interfaces.IBreadcrumbs)
    component.adapts(None, IHTTPRequest)

    def __call__(self, maxlength=None):
        context = self.context
        request = self.request
        if proxy.sameProxiedObjects(context, request.getVirtualHostRoot()):
            base = ()
        else:
            container = getattr(context, '__parent__', None)
            if container is None:
                raise TypeError, INSUFFICIENT_CONTEXT
            view = component.getMultiAdapter(
                (container, request), interfaces.IBreadcrumbs)
            base = tuple(view(maxlength))
        name_gen = component.getMultiAdapter(
            (context, request), interfaces.IDisplayNameGenerator)
        url = absoluteURL(context, request)
        return base + (
            {"name_gen": name_gen, "url": url, "name": name_gen(maxlength),
             "object": context},)

class TerminalBreadcrumbs(BrowserView):
    interface.implementsOnly(interfaces.IBreadcrumbs)
    component.adapts(IContainmentRoot, IHTTPRequest) # may adapt others

    def __call__(self, maxlength=None):
        context = self.context
        request = self.request
        name_gen = component.getMultiAdapter(
            (context, request), interfaces.IDisplayNameGenerator)
        url = absoluteURL(context, request)
        return ({"name_gen": name_gen, "url": url, "name": name_gen(maxlength),
                 "object": context},)

class HiddenBreadcrumbs(BrowserView):
    """Breadcrumbs for an object that doesn't want to be in the breadcrumbs"""
    interface.implementsOnly(interfaces.IBreadcrumbs)
    component.adapts(None, IHTTPRequest)

    def __call__(self, maxlength=None):
        context = self.context
        request = self.request
        if proxy.sameProxiedObjects(context, request.getVirtualHostRoot()):
            base = ()
        else:
            container = getattr(context, '__parent__', None)
            if container is None:
                raise TypeError, INSUFFICIENT_CONTEXT
            view = component.getMultiAdapter(
                (container, request), interfaces.IBreadcrumbs)
            base = tuple(view(maxlength))
        return base

@component.adapter(zope.location.interfaces.ILocation, IHTTPRequest)
@interface.implementer(interface.Interface)
def breadcrumbs(context, request):
    "breadcrumbs; unlimited display name length for each traversed object"
    return component.getMultiAdapter(
        (context, request), interfaces.IBreadcrumbs)()

@component.adapter(zope.location.interfaces.ILocation, IHTTPRequest)
@interface.implementer(interface.Interface)
def breadcrumbs20char(context, request):
    "breadcrumbs; display name length limited to 20 characters for each object"
    return component.getMultiAdapter(
        (context, request), interfaces.IBreadcrumbs)(20)



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################  

"""I18N support for the site package.

This defines a `MessageFactory` for the I18N domain for the
`zc.displayname` package.  This is normally used with this import::

  from i18n import _

The factory is then used normally.  Two examples::

  text = _('some internationalized text')
  text = _('helpful-descriptive-message-id', 'default text')
"""
__docformat__ = "reStructuredText"


from zope import i18nmessageid

MessageFactory = _ = i18nmessageid.MessageFactory("zc.displayname")



from setuptools import setup, find_packages

setup(
    name = "zc.displayname",
    version = "1.1",
    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = ['zc'],
    install_requires = [
    'setuptools',
    'zope.app.container',
    'zope.app.pagetemplate',
    'zope.component',
    'zope.dublincore',
    'zope.i18n',
    'zope.i18nmessageid',
    'zope.interface',
    'zope.location',
    'zope.proxy',
    'zope.publisher',
    'zope.security',
    'zope.testing',
    'zope.traversing',
    ],
    zip_safe=False,
    author='Zope Project',
    author_email='zope3-dev@zope.org',
    description=open("README.txt").read(),
    long_description=open("src/zc/displayname/adapters.txt").read(),
    license='ZPL 2.1',
    keywords="zope zope3",
    )



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################  

"""site interfaces

$Id: interfaces.py 3161 2005-07-27 21:49:58Z fred $
"""
from zope import interface

class IDisplayNameGenerator(interface.Interface):
    def __call__(maxlength=None):
        """return a display name for this object.
        
        if length is None, length is not constrained.  If maxlength is an
        integer, the display may be up to that number of characters.
        
        Implementations may refuse to support maxlengths smaller than a
        minimum value by raising ValueError; raise ValueError if maxlength < 0.
        
        Depending on the type of request for which the name generator is used,
        the display name may include (well-formed) XML such as XHTML.
        Name generators registered for IRequest should not include any such
        extra content.  If a display name generator for an IBrowserRequest
        has a maxlength, the maxlength should not include the length of the
        XML tags.
        """

class IBreadcrumbs(interface.Interface):
    def __call__(maxlength=None):
        """return a sequence of dictionaries with breadcrumb information.
        
        Each dictionary has four keys: name, url, object, and name_gen.
        
        name: a display name for the breadcrumbs, truncated with maxlength
              as passed to an IDisplayNameGenerator.
        
        url: the traversed url for the object.
        
        object: the object (available for getting other information such as
                an icon).
        
        name_gen: an IDisplayNameGenerator for the object.
        
        Objects are free to not include themselves in breadcrumbs.
        
        Breadcrumbs stop at the virtual host root, if any.
        """
